package es.um.atica.[miproyecto].security.authorization.filter;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Path;
import javax.ws.rs.core.Context;

import com.sun.jersey.api.core.HttpContext;
import com.sun.jersey.api.model.AbstractMethod;
import com.sun.jersey.api.model.AbstractResourceMethod;
import com.sun.jersey.api.model.AbstractSubResourceLocator;
import com.sun.jersey.api.model.AbstractSubResourceMethod;
import com.sun.jersey.spi.container.ContainerRequest;
import com.sun.jersey.spi.container.ContainerResponse;
import com.sun.jersey.spi.container.ResourceFilter;

import es.um.atica.jersey.api.container.filter.FundeWebResourceFilterFactory;

public class RestAuthorizationFilter extends FundeWebResourceFilterFactory {

    private RestRequestUtil restRequestUtil = new RestRequestUtil();

    public RestAuthorizationFilter( @Context HttpContext hc ) {
        super( hc );
    }

    private class ResourceMethodFilter extends AbstractResourceMethodFilter {

        public ResourceMethodFilter( AbstractResourceMethod arm ) {
            super( arm );
        }

        @Override
        public ContainerRequest filter( ContainerRequest request ) {
            Path classAnnotation = getResourceClass().getAnnotation( Path.class );
            RolesAllowed methodAnnotation = getMethod().getAnnotation( RolesAllowed.class );
            if ( classAnnotation != null && methodAnnotation != null ) {
                List<String> rolesPermitidos = Arrays.asList( methodAnnotation.value() );
                return restRequestUtil.filter( request, rolesPermitidos );
            }
            return request;
        }

        @Override
        public ContainerResponse filter( ContainerRequest request, ContainerResponse response ) {
            return response;
        }
    }

    private class SubResourceMethodFilter extends AbstractSubResourceMethodFilter {

        public SubResourceMethodFilter( AbstractSubResourceMethod asrm ) {
            super( asrm );
        }

        @Override
        public ContainerRequest filter( ContainerRequest request ) {
            Path classAnnotation = getResourceClass().getAnnotation( Path.class );
            RolesAllowed methodAnnotation = getMethod().getAnnotation( RolesAllowed.class );
            if ( classAnnotation != null && methodAnnotation != null ) {
                List<String> rolesPermitidos = Arrays.asList( methodAnnotation.value() );
                return restRequestUtil.filter( request, rolesPermitidos );
            }
            return request;
        }

        @Override
        public ContainerResponse filter( ContainerRequest request, ContainerResponse response ) {
            return response;
        }
    }

    private class SubResourceLocatorFilter extends AbstractSubResourceLocatorFilter {

        public SubResourceLocatorFilter( AbstractSubResourceLocator asrl ) {
            super( asrl );
        }

        @Override
        public ContainerRequest filter( ContainerRequest request ) {
            Path classAnnotation = getResourceClass().getAnnotation( Path.class );
            RolesAllowed methodAnnotation = getMethod().getAnnotation( RolesAllowed.class );
            if ( classAnnotation != null && methodAnnotation != null ) {
                List<String> rolesPermitidos = Arrays.asList( methodAnnotation.value() );
                return restRequestUtil.filter( request, rolesPermitidos );
            }
            return request;
        }

        @Override
        public ContainerResponse filter( ContainerRequest request, ContainerResponse response ) {
            return response;
        }
    }

    @Override
    public List<ResourceFilter> create( AbstractMethod am ) {
        if ( am instanceof AbstractSubResourceMethod ) {
            return Collections.<ResourceFilter> singletonList( new SubResourceMethodFilter(
                                                                                            ( AbstractSubResourceMethod ) am ) );
        }
        else if ( am instanceof AbstractResourceMethod ) {
            return Collections.<ResourceFilter> singletonList( new ResourceMethodFilter( ( AbstractResourceMethod ) am ) );
        }
        else if ( am instanceof AbstractSubResourceLocator ) {
            return Collections.<ResourceFilter> singletonList( new SubResourceLocatorFilter(
                                                                                             ( AbstractSubResourceLocator ) am ) );
        }
        else {
            return null;
        }
    }

}
