package es.um.atica.[miproyecto].security.authorization.filter;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;

import org.apache.log4j.Logger;
import org.jboss.seam.Component;

import com.sun.jersey.core.util.Base64;
import com.sun.jersey.spi.container.ContainerRequest;

import es.um.atica.[miproyecto].security.filter.RestRequestFilter;
import es.um.atica.[miproyecto].security.authentication.RestAuthenticationProvider;

public class RestRequestUtil {

    private Logger log = Logger.getLogger( RestRequestFilter.class );
    private final String CABECERA_AUTORIZACION = "Authorization";
    private final String NO_AUTORIZADO = "No autorizado";

    /**
     * Filtra la peticion REST y segun la URL accedida si contiene "private" busca el par usuario:token en la cabecera
     * de autenticacion y valida, si es "public" devuelve la pagina solicitada
     */
    public ContainerRequest filter( ContainerRequest request, List<String> rolesPermitidos ) {
        log.debug( "Entro en el filtro" );
        RestAuthenticationProvider provider = this.getAuthenticationProvider();

        String cabeceraAutorizacion = request.getHeaderValue( CABECERA_AUTORIZACION );
        if ( request.getPath().contains( "private/" ) ) {
            // Si no tiene cabecera de autorizacion no autorizamos
            if ( cabeceraAutorizacion == null ) {
                this.denegarConexion();
                return null;
            }
            String[] valoresCabecera = Base64.base64Decode( cabeceraAutorizacion ).split( ":" );
            // Si en la cabecera de autorizacion no van los dos parametros (nombre y token) no autorizamos
            if ( valoresCabecera.length != 2 ) {
                this.denegarConexion();
                return null;
            }
            String usuario = valoresCabecera[0];
            String token = valoresCabecera[1];
            // Si no pasa la autenticacion del token no autorizamos
            if ( !provider.authenticanteToken( usuario, token ) ) {
                this.denegarConexion();
                return null;
            }
            else {
                //Comprobamos si el usuario tiene uno de los roles permitidos para ese metodo
                //FIXME: Carga roles usuario del usuario ya logueado
                List<String>rolesUsuario = new ArrayList<String>();
                
                if ( rolesPermitidos != null && !rolesPermitidos.isEmpty() ) {
                    boolean rolEncontrado = false;
                    for(String rolUsuairo: rolesUsuario){
                        if(rolesPermitidos.contains( rolUsuairo )){
                            rolEncontrado=true;
                            break;
                        }
                    }
                    if(!rolEncontrado){
                        denegarConexion();
                        return null;
                    }
                }

                // Ha pasado la autenticacion, por lo que se lanza la peticion
                return request;
            }
        }
        else {
            // Es una peticion normal que no va por /private por lo que se lanza sin validar
            return request;
        }

    }

    /**
     * Termina la peticion entrante contestando con no autorizado
     */
    private void denegarConexion() {
        ResponseBuilder builder = null;
        builder = Response.status( Response.Status.UNAUTHORIZED ).entity( NO_AUTORIZADO );
        throw new WebApplicationException( builder.build() );
    }

    private RestAuthenticationProvider getAuthenticationProvider() {
        return ( RestAuthenticationProvider ) Component.getInstance( "restAuthenticationProvider", true );
    }

}
